@@ -127,12 +127,12 @@ class map_objects(object):
                 self.cb_gridspec,
                 self.ax_cb,
                 self.ax_cb_plot,
-                "vertical" if self._m._orientation == "horizontal" else "horizontal",
+                self._m._orientation,
             ]
         else:
             _, ax_cb, ax_cb_plot, orientation, _ = cb
 
-        if orientation == "horizontal":
+        if orientation == "vertical":
             pcb = ax_cb.get_position()
             pcbp = ax_cb_plot.get_position()
             if pos is None:
@@ -150,7 +150,7 @@ class map_objects(object):
                 [pos[0], pos[1] + hcb, pos[2], hp],
             )
 
-        elif orientation == "vertical":
+        elif orientation == "horizontal":
             pcb = ax_cb.get_position()
             pcbp = ax_cb_plot.get_position()
             if pos is None:
@@ -542,6 +542,138 @@ class classify_specs(object):
         )
 
 
+class _NaturalEarth_presets:
+    def __init__(self, m):
+        self._m = m
+
+    @property
+    def coastline(self):
+        """
+        Add a coastline to the map.
+
+        All provided arguments are passed to `m.add_feature`
+        (and further to `m.add_gdf` if geopandas is available)
+
+        The default args are:
+
+        - fc="none", ec="k", zorder=100
+        """
+        return self._feature(
+            self._m, "physical", "coastline", fc="none", ec="k", zorder=100
+        )
+
+    @property
+    def ocean(self):
+        """
+        Add ocean-coloring to the map.
+
+        All provided arguments are passed to `m.add_feature`
+        (and further to `m.add_gdf` if geopandas is available)
+
+        The default args are:
+
+        - fc=(0.59375, 0.71484375, 0.8828125), ec="none", zorder=0, reproject="cartopy"
+        """
+
+        # convert color to hex to avoid issues with geopandas
+        color = rgb2hex(cfeature.COLORS["water"])
+        return self._feature(
+            self._m,
+            "physical",
+            "ocean",
+            fc=color,
+            ec="none",
+            zorder=-1,
+            reproject="cartopy",
+        )
+
+    @property
+    def land(self):
+        """
+        Add a land-coloring to the map.
+
+        All provided arguments are passed to `m.add_feature`
+        (and further to `m.add_gdf` if geopandas is available)
+
+        The default args are:
+
+        - fc=(0.9375, 0.9375, 0.859375), ec="none", zorder=0
+
+        """
+
+        # convert color to hex to avoid issues with geopandas
+        color = rgb2hex(cfeature.COLORS["land"])
+
+        return self._feature(
+            self._m,
+            "physical",
+            "land",
+            fc=color,
+            ec="none",
+            zorder=-1,
+        )
+
+    @property
+    def countries(self):
+        """
+        Add country-boundaries to the map.
+
+        All provided arguments are passed to `m.add_feature`
+        (and further to `m.add_gdf` if geopandas is available)
+
+        The default args are:
+
+        - fc="none", ec=".5", lw=0.5, zorder=99
+
+        """
+
+        return self._feature(
+            self._m,
+            "cultural",
+            "admin_0_countries",
+            fc="none",
+            ec=".5",
+            lw=0.5,
+            zorder=99,
+        )
+
+    class _feature:
+        def __init__(self, m, category, name, **kwargs):
+            self._m = m
+            self.category = category
+            self.name = name
+
+            self.kwargs = kwargs
+
+            self.feature = getattr(
+                getattr(self._m.add_feature, f"{self.category}_10m"), self.name
+            )
+
+            add_params = """
+            Other Parameters
+            ----------------
+            scale : str
+                Set the scale of the feature preset ("10m", "50m" or "110m")
+                The default is "50m"
+            """
+
+            self.__doc__ = combdoc(
+                f"PRESET using {kwargs} ", self.feature.__doc__, add_params
+            )
+
+        def __call__(self, scale="50m", **kwargs):
+            k = dict(**self.kwargs)
+            k.update(kwargs)
+
+            self.feature = getattr(
+                getattr(self._m.add_feature, f"{self.category}_{scale}"), self.name
+            )
+
+            self.__doc__ = self.feature.__doc__
+
+            return self.feature(**k)
+
+
 class NaturalEarth_features(object):
     """
     Interface to the feature-layers provided by NaturalEarth
@@ -602,7 +734,7 @@ class NaturalEarth_features(object):
         - "land" - beige land coloring
         - "countries" - gray country boarder lines
         """
-        return self._presets(self._m)
+        return _NaturalEarth_presets(self._m)
 
     def __getitem__(self, key):
         return getattr(self, key)
@@ -664,8 +796,12 @@ class NaturalEarth_features(object):
                     Parameters
                     ----------
 
-                    layer : int, optional
-                        The EOmaps layer-number at which the feature is drawn.
+                    layer : int, str or None, optional
+                        The name of the layer at which map-features are plotted.
+
+                        - If "all": the corresponding feature will be added to ALL layers
+                        - If None, the layer of the parent object is used.
+
                         The default is None.
 
                     Note
@@ -754,8 +890,12 @@ class NaturalEarth_features(object):
 
                         (facecolor, edgecolor, etc.)
 
-                    layer : int, optional
-                        The EOmaps layer-number at which the feature is drawn.
+                    layer : int, str or None, optional
+                        The name of the layer at which map-features are plotted.
+
+                        - If "all": the corresponding feature will be added to ALL layers
+                        - If None, the layer of the parent object is used.
+
                         The default is None.
 
 
@@ -821,92 +961,6 @@ class NaturalEarth_features(object):
                 gdf.set_crs(ccrs.PlateCarree(), inplace=True, allow_override=True)
                 return gdf
 
-    class _presets:
-        def __init__(self, m):
-            self._m = m
-
-        @property
-        def coastline(self):
-            return self._feature(
-                self._m, "physical", "coastline", fc="none", ec="k", zorder=100
-            )
-
-        @property
-        def ocean(self):
-            # convert color to hex to avoid issues with geopandas
-            color = rgb2hex(cfeature.COLORS["water"])
-            return self._feature(
-                self._m,
-                "physical",
-                "ocean",
-                fc=color,
-                ec="none",
-                zorder=0,
-                reproject="cartopy",
-            )
-
-        @property
-        def land(self):
-            # convert color to hex to avoid issues with geopandas
-            color = rgb2hex(cfeature.COLORS["land"])
-
-            return self._feature(
-                self._m,
-                "physical",
-                "land",
-                fc=color,
-                ec="none",
-                zorder=0,
-            )
-
-        @property
-        def countries(self):
-            return self._feature(
-                self._m,
-                "cultural",
-                "admin_0_countries",
-                fc="none",
-                ec=".5",
-                lw=0.5,
-                zorder=100,
-            )
-
-        class _feature:
-            def __init__(self, m, category, name, **kwargs):
-                self._m = m
-                self.category = category
-                self.name = name
-
-                self.kwargs = kwargs
-
-                self.feature = getattr(
-                    getattr(self._m.add_feature, f"{self.category}_10m"), self.name
-                )
-
-                add_params = """
-                Other Parameters
-                ----------------
-                scale : str
-                    Set the scale of the feature preset ("10m", "50m" or "110m")
-                    The default is "50m"
-                """
-
-                self.__doc__ = combdoc(
-                    f"PRESET using {kwargs} ", self.feature.__doc__, add_params
-                )
-
-            def __call__(self, scale="50m", **kwargs):
-                k = dict(**self.kwargs)
-                k.update(kwargs)
-
-                self.feature = getattr(
-                    getattr(self._m.add_feature, f"{self.category}_{scale}"), self.name
-                )
-
-                self.__doc__ = self.feature.__doc__
-
-                return self.feature(**k)
-
 
 # avoid defining containers if import is not OK
 if not _import_OK:
@@ -1092,6 +1146,51 @@ else:
             WMS.__doc__ = type(self).ESA_WorldCover.__doc__
             return WMS
 
+        @property
+        @lru_cache()
+        def GEBCO(self):
+            """
+            Global ocean & land terrain models
+            https://www.gebco.net/
+
+            GEBCO aims to provide the most authoritative, publicly available bathymetry
+            data sets for the world’s oceans.
+
+            GEBCO's current gridded bathymetric data set, the GEBCO_2021 Grid, is a
+            global terrain model for ocean and land, providing elevation data, in
+            meters, on a 15 arc-second interval grid. It is accompanied by a Type
+            Identifier (TID) Grid that gives information on the types of source data
+            that the GEBCO_2021 Grid is based.
+
+            For this release, we are making available a version of the grid with
+            under-ice topography/bathymetry information for Greenland and Antarctica.
+
+
+            Note
+            ----
+            **LICENSE-info (without any warranty for correctness!!)**
+
+            The GEBCO Grid is placed in the public domain and may be used free of charge.
+
+            If imagery from the WMS is included in web sites, reports and digital and
+            printed imagery then we request that the source of the data set is
+            acknowledged and be of the form
+
+            "Imagery reproduced from the GEBCO_2021 Grid, GEBCO Compilation Group (2021)
+            GEBCO 2021 Grid (doi:10.5285/c6612cbe-50b3-0cff-e053-6c86abc09f8f)"
+
+            (check: https://www.gebco.net/ for full details)
+
+            """
+            WMS = _WebServiec_collection(
+                m=self._m,
+                service_type="wms",
+                url="https://www.gebco.net/data_and_products/gebco_web_services/web_map_service/mapserv?request=getcapabilities&service=wms&version=1.1.1",
+            )
+
+            WMS.__doc__ = type(self).GEBCO.__doc__
+            return WMS
+
         @property
         @lru_cache()
         def NASA_GIBS(self):
@@ -1321,7 +1420,9 @@ else:
                     )
 
                     self.stamen_watercolor = _xyz_tile_service(
-                        self._m, "http://c.tile.stamen.com/watercolor/{z}/{x}/{y}.jpg"
+                        self._m,
+                        "http://c.tile.stamen.com/watercolor/{z}/{x}/{y}.jpg",
+                        maxzoom=18,
                     )
 
                     self.stamen_terrain = _xyz_tile_service(
@@ -1894,32 +1995,18 @@ else:
             WMS.__doc__ = type(self).Austria.__doc__
             return WMS
 
-        def get_service(self, url, service_type="wms", rest_API=False):
+        def get_service(self, url, service_type="wms", rest_API=False, maxzoom=19):
             """
-            Get a object that can be used to add WMS or WMTS services based on
+            Get a object that can be used to add WMS, WMTS or XYZ services based on
             a GetCapabilities-link or a link to a ArcGIS REST API
 
-            Examples (WMS):
-            - Copernicus Global Land Mosaics for Austria, Germany and Slovenia
-              from Sentinel-2
+            The general usage is as follows (see examples below for more details):
 
-              - https://land.copernicus.eu/imagery-in-situ/global-image-mosaics/
-              >>> "https://s2gm-wms.brockmann-consult.de/cgi-bin/qgis_mapserv.fcgi?MAP=/home/qgis/projects/s2gm-wms_mosaics_vrt.qgs&service=WMS&request=GetCapabilities&version=1.1.1"
-            - Web Map Services of the city of Vienna (Austria)
-
-              - https://www.data.gv.at/katalog/dataset/stadt-wien_webmapservicewmsgeoserverwien
-              >>> "https://data.wien.gv.at/daten/geo?version=1.3.0&service=WMS&request=GetCapabilities"
-
-            Examples (WMTS):
-            - WMTS service for NASA GIBS datasets
-
-              >>> https://gibs.earthdata.nasa.gov/wmts/epsg4326/all/1.0.0/WMTSCapabilities.xml
-
-            Examples (rest_API):
-            - Interface to the ArcGIS REST Services Directory for the
-              Austrian Federal Institute of Geology (Geologische Bundesanstalt)
-              - https://www.geologie.ac.at/services/web-services
-              >>> "https://gisgba.geologie.ac.at/arcgis/rest/services"
+            >>> m = Maps()
+            >>> s = m.add_wms.get_service(...)
+            >>> wms = s.add_layer.<...>
+            >>> wms.set_extent_to_bbox() # set the extent of the map to the wms-extent
+            >>> wms(transparent=True) # add the service to the map
 
             Parameters
             ----------
@@ -1927,9 +2014,35 @@ else:
                 The service-url
             service_type: str
                 The type of service (either "wms" or "wmts")
+
+                - "wms" : `url` represents a link to a GetCapabilities.xml file for a
+                  WebMap service
+                - "wmts" : same as "wms" but for a WebMapTile service
+                - "xyz" : A direct link to a xyz-TileServer
+                  (the name of the layer is set to "xyz_layer")
+
+                  The url can be provided either as a string of the form:
+
+                  >>> "https://.../{z}/{x}/{y}.png"
+
+                  Or (for non-standard urls) as a function with the following
+                  call-signature:
+
+                  >>> def url(x, y, z):
+                  >>>     return "the url with x, y, z replaced by the arguments"
+
+                See the examples below for more details on common use-cases.
+
             rest_API : bool, optional
+                ONLY relevant if service_type is either "wms" or "wmts"!
+
                 Indicator if a GetCapabilities link (True) or a link to a
                 rest-API is provided (False). The default is False
+            maxzoom : int
+                ONLY relevant if service_type="xyz"!
+
+                The maximum zoom-level available (to avoid http-request errors) for too
+                high zoom levels. The default is 19.
 
             Returns
             -------
@@ -1937,17 +2050,72 @@ else:
                 An object that behaves just like `m.add_wms.<service>`
                 and provides easy-access to available WMS layers
 
+            Examples
+            --------
+
+            WMS Example:
+
+            - Copernicus Global Land Mosaics for Austria, Germany and Slovenia
+              from Sentinel-2
+
+              - https://land.copernicus.eu/imagery-in-situ/global-image-mosaics/
+              >>> url = "https://s2gm-wms.brockmann-consult.de/cgi-bin/qgis_mapserv.fcgi?MAP=/home/qgis/projects/s2gm-wms_mosaics_vrt.qgs&service=WMS&request=GetCapabilities&version=1.1.1"
+              >>> s = m.add_wms.get_service(url, "wms")
+
+            - Web Map Services of the city of Vienna (Austria)
+
+              >>> url = "https://data.wien.gv.at/daten/geo?version=1.3.0&service=WMS&request=GetCapabilities"
+              >>> s = m.add_wms.get_service(url, "wms")
+
+            WMTS Example:
+
+            - WMTS service for NASA GIBS datasets
+
+              >>> url = https://gibs.earthdata.nasa.gov/wmts/epsg4326/all/1.0.0/WMTSCapabilities.xml
+              >>> s = m.add_wms.get_service(url, "wmts")
+
+            Rest API Example:
+
+            - Interface to the ArcGIS REST Services Directory for the
+              Austrian Federal Institute of Geology (Geologische Bundesanstalt)
+
+              >>> url = "https://gisgba.geologie.ac.at/arcgis/rest/services"
+              >>> s = m.add_wms.get_service(url, "wms", rest_API=True)
+
+            XYZ Example:
+
+            - OpenStreetMap Tiles (https://wiki.openstreetmap.org/wiki/Tiles)
+
+              >>> url = r"https://tile.openstreetmap.org/{z}/{x}/{y}.png"
+              >>> s = m.add_wms.get_service(url, "xyz")
+
+
+              >>> def url(x, y, z):
+              >>>     return rf"https://tile.openstreetmap.org/{z}/{x}/{y}.png"
+              >>> s = m.add_wms.get_service(url, "xyz")
+
             """
+            if service_type == "xyz":
+                if rest_API:
+                    print(
+                        "EOmaps: rest_API=True is not supported for service_type='xyz'"
+                    )
+
+                s = _xyz_tile_service(self._m, url, maxzoom=maxzoom)
+                service = SimpleNamespace(add_layer=SimpleNamespace(xyz_layer=s))
 
-            if rest_API:
-                service = REST_API_services(
-                    m=self._m,
-                    url=url,
-                    name="custom_service",
-                    service_type=service_type,
-                )
             else:
-                service = _WebServiec_collection(self._m, service_type="wms", url=url)
+                if rest_API:
+                    service = REST_API_services(
+                        m=self._m,
+                        url=url,
+                        name="custom_service",
+                        service_type=service_type,
+                    )
+                else:
+                    service = _WebServiec_collection(
+                        self._m, service_type="wms", url=url
+                    )
 
             return service
 
