@@ -85,14 +85,23 @@ class shapes(object):
             if not hasattr(m, "_props"):
                 m._props = m._prepare_data()
         else:
+            if isinstance(radius, (list, np.ndarray)):
+                radiusx = radiusy = tuple(radius)
             # get manually specified radius (e.g. if radius != "estimate")
-            if isinstance(radius, (list, tuple, np.ndarray)):
+            elif isinstance(radius, tuple):
                 radiusx, radiusy = radius
             elif isinstance(radius, (int, float, np.number)):
-                radiusx = radius
-                radiusy = radius
+                radiusx = radiusy = radius
 
             # we need an immutuable object for the lru_cache!
+            # (if radius is passed as list we need to convert it to a hashable object)
+            try:
+                hash(radiusx)
+                hash(radiusy)
+            except TypeError:
+                radiusx = tuple(radiusx)
+                radiusy = tuple(radiusy)
+
             radius = tuple((radiusx, radiusy))
 
         return shapes._get_radius_cache(
@@ -107,6 +116,7 @@ class shapes(object):
         if (isinstance(radius, str) and radius == "estimate") or radius is None:
             print("EOmaps: estimating radius...")
             radiusx, radiusy = shapes._estimate_radius(m, radius_crs)
+            print(f"EOmaps: The estimated radius is: {radiusx:.4f}")
         else:
             radiusx, radiusy = radius
 
@@ -114,8 +124,6 @@ class shapes(object):
             radiusx = radiusx * buffer
             radiusy = radiusy * buffer
 
-        print(f"EOmaps: The estimated radius is: {radiusx:.4f}")
-
         return (radiusx, radiusy)
 
     @staticmethod
@@ -126,12 +134,8 @@ class shapes(object):
             in_tree = cKDTree(
                 np.stack(
                     [
-                        m._props["xorig"][m._props["mask"]][
-                            : m.set_shape.radius_estimation_range
-                        ],
-                        m._props["yorig"][m._props["mask"]][
-                            : m.set_shape.radius_estimation_range
-                        ],
+                        m._props["xorig"][: m.set_shape.radius_estimation_range],
+                        m._props["yorig"][: m.set_shape.radius_estimation_range],
                     ],
                     axis=1,
                 ),
@@ -146,12 +150,8 @@ class shapes(object):
             tree = cKDTree(
                 np.stack(
                     [
-                        m._props["x0"][m._props["mask"]][
-                            : m.set_shape.radius_estimation_range
-                        ],
-                        m._props["y0"][m._props["mask"]][
-                            : m.set_shape.radius_estimation_range
-                        ],
+                        m._props["x0"][: m.set_shape.radius_estimation_range],
+                        m._props["y0"][: m.set_shape.radius_estimation_range],
                     ],
                     axis=1,
                 ),
@@ -323,7 +323,6 @@ class shapes(object):
 
             # special treatment of array input to properly mask values
             array = kwargs.pop("array", None)
-            # xs, ys, mask = props["xs"], props["ys"], props["mask"]
 
             # only plot polygons if they contain 2 or more vertices
             vertmask = np.count_nonzero(mask, axis=0) > 2
@@ -476,34 +475,26 @@ class shapes(object):
             [rx, ry] = radius
             # transform corner-points
             if radius_crs == crs:
-                pr = t_in_plot.transform(x, y + ry)
-                pl = t_in_plot.transform(x, y - ry)
-                d = np.ma.masked_invalid(pl) - np.ma.masked_invalid(pr)
-                theta = np.arctan(d[1] / d[0]) - (np.pi / 2)
-
+                theta = np.full_like(x, 0)
                 xs, ys = self._calc_ellipse_points(
                     x,
                     y,
                     np.full_like(x, rx, dtype=float),
                     np.full_like(x, ry, dtype=float),
-                    theta,
+                    np.full_like(x, 0),
                     n=n,
                 )
                 xs, ys = np.ma.masked_invalid((xs, ys), copy=False)
                 xs, ys = np.ma.masked_invalid(t_in_plot.transform(xs, ys), copy=False)
             else:
                 p = t_in_radius.transform(x, y)
-                pr = t_radius_plot.transform(p[0], p[1] + y)
-                pl = t_radius_plot.transform(p[0], p[1] - y)
-
-                d = np.ma.masked_invalid(pl) - np.ma.masked_invalid(pr)
-                theta = np.arctan(d[1] / d[0]) - (np.pi / 2)
+                theta = np.full_like(x, 0)
                 xs, ys = self._calc_ellipse_points(
                     p[0],
                     p[1],
                     np.full_like(x, rx, dtype=float),
                     np.full_like(x, ry, dtype=float),
-                    theta,
+                    np.full_like(x, 0),
                     n=n,
                 )
 
@@ -1219,7 +1210,6 @@ class shapes(object):
                 aggregator=self.aggregator,
                 shade_hook=self.shade_hook,
                 agg_hook=self.agg_hook,
-                glyph=self.glyph,
             )
 
         @property
@@ -1281,7 +1271,8 @@ class shapes(object):
             if agg_hook is None:
                 pass
 
-            glyph = ds.glyphs.QuadMeshRectilinear("x", "y", "val")
+            # this might be changed by m._shade_raster depending on the dataset-shape
+            glyph = None
 
             from . import MapsGrid  # do this here to avoid circular imports!
 
@@ -1301,7 +1292,6 @@ class shapes(object):
                 aggregator=self.aggregator,
                 shade_hook=self.shade_hook,
                 agg_hook=self.agg_hook,
-                glyph=self.glyph,
             )
 
         @property
