@@ -116,7 +116,7 @@ class _click_callbacks(object):
         val_precision=4,
         permanent=False,
         text=None,
-        layer=10,
+        zorder=10,
         **kwargs,
     ):
         """
@@ -152,11 +152,13 @@ class _click_callbacks(object):
                 >>>     return "the string to print"
 
             The default is None.
-        layer : int
-            The layer-level on which to draw the artist.
-            (First layer 0 is drawn, then layer 1 on top then layer 2 etc...)
-            The default is 10.
-        **kwargs
+        zorder : int or float
+            The zorder of the artist. (e.g. the drawing-order)
+            For details, have a look at:
+
+            - https://matplotlib.org/stable/gallery/misc/zorder_demo.html
+
+        kwargs
             kwargs passed to matplotlib.pyplot.annotate(). The default is:
 
             >>> dict(xytext=(20, 20),
@@ -250,8 +252,8 @@ class _click_callbacks(object):
                 else:
                     self.permanent_annotations.append(annotation)
 
-            if layer is not None:
-                self.m.BM.add_artist(annotation, layer=layer)
+            annotation.set_zorder(zorder)
+            self.m.BM.add_artist(annotation)
 
             annotation.set_visible(True)
             annotation.xy = pos
@@ -307,7 +309,7 @@ class _click_callbacks(object):
         buffer=1,
         permanent=True,
         n=20,
-        layer=10,
+        zorder=10,
         **kwargs,
     ):
         """
@@ -349,10 +351,11 @@ class _click_callbacks(object):
         n : int
             The number of points to calculate for the shape.
             The default is 20.
-        layer : int
-            The layer-level on which to draw the artist.
-            (First layer 0 is drawn, then layer 1 on top then layer 2 etc...)
-            The default is 10.
+        zorder : int or float
+            The zorder of the artist. (e.g. the drawing-order)
+            For details, have a look at:
+
+            - https://matplotlib.org/stable/gallery/misc/zorder_demo.html
         kwargs :
             kwargs passed to the matplotlib patch.
             (e.g. `facecolor`, `edgecolor`, `linewidth`, `alpha` etc.)
@@ -392,7 +395,7 @@ class _click_callbacks(object):
         else:
             pos_crs = "out"
 
-        if radius == "pixel":
+        if isinstance(radius, str) and radius == "pixel":
             pixelQ = True
             if not hasattr(self.m.shape, "radius"):
                 print(
@@ -405,8 +408,10 @@ class _click_callbacks(object):
             pixelQ = False
 
         # get manually specified radius (e.g. if radius != "estimate")
-        if isinstance(radius, (list, tuple)):
+        if isinstance(radius, list):
             radius = [i * buffer for i in radius]
+        elif isinstance(radius, tuple):
+            radius = tuple([i * buffer for i in radius])
         elif isinstance(radius, (int, float)):
             radius = radius * buffer
 
@@ -452,8 +457,8 @@ class _click_callbacks(object):
         else:
             self._temporary_artists.append(marker)
 
-        if layer is not None:
-            self.m.BM.add_artist(marker, layer)
+        marker.set_zorder(zorder)
+        self.m.BM.add_artist(marker)
 
         return marker
 
@@ -471,21 +476,15 @@ class _click_callbacks(object):
     def _mark_cleanup(self):
         self.clear_markers()
 
-    def peek_layer(self, layer=1, how="left", **kwargs):
+    def peek_layer(self, layer=1, how="left", overlay=False, **kwargs):
         """
         Swipe between data- or WebMap layers or peek a layers through a rectangle.
 
         Parameters
         ----------
-        layer : int
-            The layer-number you want to peek at.
-
-            Note:
-            You must draw something on the layer first! (Most EOmaps functions
-            that add features to a map support a `layer` argument that
-            lets you specify the layer at which the object is drawn.)
-
-                >>> m.plot_map(layer=1)
+        layer : int, str or list
+            - if int or str: The name of the layer you want to peek at.
+            - if list: A list of layer-names to peek at.
 
         how : str , float or tuple, optional
             The method you want to visualize the second layer.
@@ -499,10 +498,27 @@ class _click_callbacks(object):
                   as percentage of the axis-size (0-1)
 
             The default is "left".
+        overlay : bool, optional
+            Indicator if only the selected layers should be shown (False) or if the
+            layers should be used as an "overlay" on top of the current layer (True).
+            The default is False.
         **kwargs :
             additional kwargs passed to a rectangle-marker.
             the default is `(fc="none", ec="k", lw=1)`
 
+        Note
+        ----
+        You must draw something on the layer first!
+
+        To assign a layer to an object, either use the `layer=...` argument when
+        adding objects (e.g. `m.plot_map(layer=1)`), or use a new Maps-layer via
+
+        >>> m = Maps()
+        >>> m2 = m.new_layer(layer="the layer name")
+        >>> # now all artists added with m2 will be added to the layer
+        >>> # "the layer name" (if not explicitly specified otherwise)
+        >>> m2.plot_map()
+        >>> m.peek_layer(layer="the layer name")
         """
         ID, pos, val, ind, picker_name = self._popargs(kwargs)
 
@@ -544,10 +560,9 @@ class _click_callbacks(object):
             x1m, y1m = ax.transData.inverted().transform((x0 + blitw, y0 + blith))
             w, h = abs(x1m - x0m), abs(y1m - y0m)
 
-            self.mark(
+            marker = self.mark(
                 pos=((x0m + x1m) / 2, (y0m + y1m) / 2),
                 radius_crs="out",
-                layer=1,
                 shape="rectangles",
                 radius=(w / 2, h / 2),
                 permanent=False,
@@ -596,10 +611,9 @@ class _click_callbacks(object):
             )
             w, h = abs(x1m - x0m), abs(y1m - y0m)
 
-            self.mark(
+            marker = self.mark(
                 pos=pos,
                 radius_crs="out",
-                layer=1,
                 shape="rectangles",
                 radius=(w / 2, h / 2),
                 permanent=False,
@@ -608,9 +622,16 @@ class _click_callbacks(object):
         else:
             raise TypeError(f"EOmaps: {how} is not a valid peek method!")
 
-        self.m.BM._after_restore_actions.append(
-            self.m.BM._get_restore_bg_action(layer, (x0, y0, blitw, blith))
-        )
+        self.m.BM._artists_to_clear["on_layer_change"].append(marker)
+
+        if overlay:
+            self.m.BM._after_restore_actions.append(
+                self.m.BM._get_overlay_bg_action(layer, (x0, y0, blitw, blith))
+            )
+        else:
+            self.m.BM._after_restore_actions.append(
+                self.m.BM._get_restore_bg_action(layer, (x0, y0, blitw, blith))
+            )
 
     def load(
         self, database=None, load_method="load_fit", load_multiple=False, **kwargs
@@ -847,7 +868,7 @@ class dynamic_callbacks:
     """
 
     def __init__(self, m):
-        self.m = m
+        self._m = m
 
     def indicate_extent(self, m=None, **kwargs):
         """
@@ -892,7 +913,7 @@ class dynamic_callbacks:
                 args = dict(n=100, fc="none", ec="r")
                 args.update(kwargs)
 
-                self._extent_marker = self.m.add_marker(
+                self._extent_marker = self._m.add_marker(
                     xy=((x0 + x1) / 2, (y0 + y1) / 2),
                     xy_crs=m.crs_plot,
                     layer=999,
